;;;; File: res-struct.lsp
;;;; SCCS Version: %W%
;;;; Contains: KS_OR result structure def.
;;;; Author: Richard Kirby (rbk)
;;;; Created: Mon Dec  2 15:33:15 1991
;;;; Updated: Thu Apr 21 17:45:58 1994 by Jeff Dalton
;;;; Release Version: %Y%
;;;; Copyright: (c) 1992, AIAI, University of Edinburgh
;;;; This material may be reproduced by or for the U.S. Government pursuant
;;;; to the copyright license under the clause at DFARS 252.227-7032
;;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

;;;; KS_OR info record.

(in-package :QA)

(use-package :MONITORS)

#+kcl
(use-package :XP)

(export '(or-info-rec-branch-1 or-info-rec-branch-n or-info-rec-actions
	  or-tree-merger))

(defstruct or-info-rec
  branch-1
  branch-n
  actions
  )

#|
The actions field is a list of lists, of the form:
(<keyword> . <args>)
where <keyword> is one of:
:LINK - with two args giving the node-ends to link in the order from, to.
:BIND - with two args giving the variable to bind, and the value to bind it to.
:RESTRICT - with two args giving the variable to restrict, and the value to
restrict it to.
:REACHIEVE - with the condition as the first argument, and the current
contributor list as the second, which can then be used to decide on the next
appropriate method to use for resatisfying the condition.
:UPDATE - Updates the contributors of the condition to be the first argument.
|#

#| OR tree merging algorithm:
Do a depth first pass through tree1, and at each node

|#

;; Takes two or-trees and merges the second into the first removing
;; inconsistencies. Destructively modifies the first tree.
#+:undef
(defun or-tree-merger (tree1 tree2)
  )

;;;; Pretty printing functions.

#+:undef
(defun pretty-print-or-info-rec (stream or-info-rec)
  (pprint-logical-block
      (stream (or-info-rec-actions or-info-rec)
	      :prefix "#<:or-actions: "
	      :suffix ">")
    (pprint-newline :fill stream)
    (loop
       (let ((action (pprint-pop)))
	 (case (car action)
	   (:LINK
	    (mon-format stream "Link ~W to ~W; "
			(second action) (third action)))
	   (:BIND
	    (mon-format stream "Bind ~W to ~W; "
			(second action) (third action)))
	   (:RESTRICT
	    (mon-format stream "Restrict ~W to not be ~W; "
			(second action) (third action)))
	   (:REACHIEVE
	    (mon-format stream "Reachieve ~W; "
			(second action)))
	   (:UPDATE
	    (mon-format stream "Update contributors to be ~W; "
			(second action))))
	 (pprint-newline :fill stream)))))

(defun pretty-print-or-info-rec (stream or-info-rec)
  (when (and (boundp '*monitor-level*)
	     (member *monitor-level* '(:one-liner :minimum)))
    ; (mon-format stream "#<or-info-rec>")
    (mon-format stream "#")		;fairly minimal
    (return-from pretty-print-or-info-rec nil))
  (mon-format stream "OR-INFO-REC:~%")
  (dolist (action (or-info-rec-actions or-info-rec))
    (case (car action)
      (:LINK
       (mon-format stream "Link ~W to ~W~%" (second action) (third action)))
      (:BIND
       (mon-format stream "Bind ~W to ~W~%" (second action) (third action)))
      (:RESTRICT
       (mon-format stream "Restrict ~W to not be ~W~%"
		   (second action) (third action)))
      (:REACHIEVE
       (mon-format stream "Reachieve ~W~%" (second action)))
      (:UPDATE
       (mon-format stream "Update contributors to be ~W~%"
		   (second action))))))

(mon-add-user-def-print-fn 'or-info-rec #'pretty-print-or-info-rec)

;;;; ---------------------------- Change History ----------------------------
;;;; (Initials) (Date of change)           (Comment)
