;;;; File: com-defs.lsp
;;; SCCS Version: %W%
;;; Contains: Definitions for parser and compiler
;;; Author: Jeff Dalton <J.Dalton@ed.ac.uk>
;;; Created: March 1992
;;; Updated: Sun Oct 16 00:19:00 1994 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1992, 1993, 1994 AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

(in-package :tf-compiler)

(defparameter *tf-language-package* (find-package :tf-language))

;;; Compilation parameters

;;; If the variable *check-syntax-only* is true, then syntax
;;; checking should be performed without other processing such
;;; as code generation.

;;; /\/: Actually, this is somewhat losing, because a number of useful
;;; checks are effectively part of code generation.  Maybe make "check-only"
;;; do all but modify the domain given to the compiler.

(defvar *syntax-check-only* nil
  "True to turn off code generation.")

(defvar *running-as-tfc* nil
  "True if running as the standalone syntax checker (tfc).")

(defvar *tf-stream* nil
  "The stream containing TF forms, when compiling from a stream.")

(defvar *include-stack* nil
  "Information saved while processing include forms.")

(defvar *input-domain* nil
  "Domain information constructed before the current call to the compiler
   and being used as a base for further compilation.")

(defvar *output-domain* nil
  "Domain information that will be the result of the current call to the
   compiler.")

(defvar *domain* nil
  "The domain information structure being processed by the compiler.")


;;; Reporting parameters

(defparameter *suppress-messages* nil)

(defparameter *progress-reports* t)

(defparameter *level-report* nil)


;;; Other globals

(defvar *new-schemas* nil
  "New schema structures as they are produced by the compiler.")

(defvar *schema-plist*
  ;; Stores random information that is not part of the parse tree.
  )

(defmacro getf-schema (prop)
  `(getf *schema-plist* ,prop))

(defparameter *schema-types*		; /\/: should be keywords?
  '(schema meta_schema process_schema meta_process_schema))

(defconstant *undef* ':undef)


;;; Schema names

(defun ->schema-name (sym)
  (string-downcase (string sym)))

(defun ->task-name (sym)
  (concatenate 'string
	       "task_"
	       (string-downcase (string sym))))

(defun same-name-p (name1 name2)
  (string= name1 name2))

(defun task-name-p (name)
  (and (stringp name)
       (eql (mismatch "task_" name) #.(length "task_"))))

;;; End
