;;;; File: control-panel.lsp
;;; SCCS Version: %W%
;;; Contains: The Lisp interface to the control panel (xuim)
;;; Author: Jeff Dalton <J.Dalton@ed.ac.uk>
;;; Created: March 1993
;;; Updated: Sat Oct 29 23:47:54 1994 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1992, 1993, AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

;;; The Lisp interface to the control panel (xuim)

;;; Note that there can't be a control panel in subr mode, because when
;;; there was nothing else to do, the pprocess-main-loop would wait for
;;; input from the control panel instead of exiting.

(in-package :oplan)

(defvar *control-io* nil)

(defun set-up-control-panel ()
  (unless (or (not (get-parameter :interactive))
	      (get-parameter :no-windows)
	      (get-parameter :subr))
    (let ((*whoami* :xuim)		;/\/: s.b. :cp
	  (*debug-io* *debug-io*)
	  (*trace-output* (make-synonym-stream '*debug-io*))
	  (*error-output* (make-synonym-stream '*debug-io*)))
      (new-pprocess *whoami*
         :specials '(*whoami* *debug-io* *trace-output* *error-output*)
         :status :run
         :run-function 'xuim-startup))))

(defun xuim-startup (self)
  (setq *control-io* (run-xuim))
  (push *control-io* (pprocess-input-streams self))
  (setq *debug-io* (pprocess-symbol-value (find-pprocess :im) '*debug-io*))
  (unless (is-init :xuim)
    (ipc-set-up-and-register-client :mcc))
  (dev-debug-msg "Xuim is running." *whoami* :information)
  ;; Equiv of (mcc-register-with-im) is sent by xuim when it starts.
  (ipc-set-event-handler 'xuim-event-handler))

(defun run-xuim ()
  (apply #'ipc-run-program
     (concat-string (get-parameter :oplan-dir) "/bin/xuim")
     (or (is-get-window-args :xuim)
	 (if (is-exec-p)
	     '("-title" "Exec Control Panel (CP)"
	       "-name" "Exec CP"
	       "-geometry" "+459+16")
	     '("-title" "O-Plan Control Panel (CP)"
	       "-name" "O-Plan CP"
	       "-geometry" "+459+16")))))

;;; The event handler:
;;;  * Takes messages from the IM via :MCC and outputs them to *control-io*.
;;;  * Takes input from *control-io* and sends it to the IM via :MCC.

(defun xuim-event-handler (self)
  (when (next-event-p self)
    (handle-message-to-xuim (next-event self)))
  (when (listen *control-io*)
    (let ((msg (read *control-io*)))
      #+kcl
        (pprocess::really-clear-whitespace *control-io*)
      (handle-message-from-xuim msg))))

;;; /\/: Both Lucid and KCL signal an error when we try to close
;;; *control-io*, presumably because xuim has terminated and (hence?)
;;; (somehow) caused at least half of the two-way stream to become
;;; closed.  In Lucid, you can normally close a stream twice, so
;;; presumably LCL isn't expecting this.

(defun handle-message-to-xuim (event)
  (let* ((msg (message-contents event))
	 (msg-list (mcc-msg->list msg)))
    ;; /\/: *print-length*, etc?
    (prin1 msg-list *control-io*)
    (force-output *control-io*)
    (case (mcc-msg-ident msg)
      (:DIE-DIE-DIE
       ;; We have to exit too.
       ; (close *control-io*)
       (terminate-pprocess)))))

(defun handle-message-from-xuim (msg-list)
  (let ((msg (list->mcc-msg msg-list)))
    (ipc-write :mcc msg)
    (case (mcc-msg-ident msg)
      (:DYING
       ;; We have to exit too
       ; (close *control-io*)
       (terminate-pprocess)))))

;;; End

  
