;;;; File: im-toplevel.lsp
;;; SCCS Version: %W%
;;; Contains: Top-level processing for the IM.
;;; Author: Richard Kirby (rbk) and Jeff Dalton
;;; Created: Thu Apr 12 11:38:24 1990
;;; Updated: Sat Oct 29 18:52:20 1994 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1992, AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

(in-package :oplan)

(use-package :monitors)

(defun im-startup ()

  ;; This is the initial function for the IM process.  After startup,
  ;; it is not called again.  Instead, im-event-handler is called
  ;; whenever a message arrives.
  
  (unless (is-init *whoami*)
    (dev-error *whoami* "is-init failed."))
  (setf (component-init-fn *pprocess*) #'im-init)
  ;; Make breaks, etc. go to the debug window.
  (x-redirect-errors :imout)
  (clear-screen *debug-io*)
  (set-up-control-panel)
  (ipc-set-event-handler 'im-event-handler))

(defun im-event-handler (self)
  (let* ((message (next-event self))
	 (route (message-route message))
	 (contents (message-contents message)))
    (ecase route
      (:MCC (mcc-controller-handler route contents))
      (:LEFTIN (apply #'handle-leftin-event contents))
      (:RIGHTIN (apply #'handle-rightin-event contents)))))

;;; /\/: We don't use the icp- message handling stuff becuase we may
;;; want to distinguish between :leftin and :rightin events.  Maybe
;;; we shouldn't care.  Maybe we should extend the ipc- code.  It may
;;; become clear in time.

(defun handle-leftin-event (message-type event-s)
  (ecase message-type
    (:EVENT
     (assert (event-p event-s))
     ;; N.B. Used to write an :AGENDA message.  :EVENT is so the AM
     ;; knows to use the agent agenda.
     (ipc-send :AGENDAIN
	       :EVENT (convert-to-agenda-entry event-s)))))

;;; /\/: Why should the IM know what an agenda-entry looks like?
;;; Sure, a KP has to know; but the IM?

(defun convert-to-agenda-entry (event)
  ;; The ident field will be set by the AM on reading on from the AGENDAIN
  ;; channel. A unique id will be assigned, of the form AE-n.
  (make-agenda-rec :trigger t
		   :body (event-body event)))

(defun handle-rightin-event (message-type &rest args)
  ;; /\/: Note that, unlike leftin events, rightins are not event structs.
  (ecase message-type
    ((:READ				;arbitrary messages from rightin
      :WORLD-EVENT)			;world events (only to exec)
     (assert (length=1 args))
     (ipc-send :AGENDAIN
	       :EVENT (make-agenda-rec :trigger t
				       :body (first args))))))

(defun im-init ()
  (clear-screen *debug-io*)
  (dev-debug :information "Initialised"))

;;; ---------------------------- Change History ----------------------------
;;; (Who)   (When) 		     (What)
;;;

