;;;; File: KS-SET-TASK.lsp
;;;; SCCS Version: %W%
;;;; Contains: Handles the initialisation of the plan state to a new task.
;;;; Author: Richard Kirby (rbk)
;;;; Created: Thu Jul 12 11:02:41 1990
;;;; Updated: Thu Jul  6 18:21:05 1995 by Jeff Dalton
;;;; Release Version: %Y%
;;;; Copyright: (c) 1992, AIAI, University of Edinburgh
;;;; This material may be reproduced by or for the U.S. Government pursuant
;;;; to the copyright license under the clause at DFARS 252.227-7032
;;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

(in-package :knowledge-source)

(defun KS-SET-TASK (ag)
  (let ((task (ag-body ag)))
    (let ((schema (ipc-with-own-copy
		    ;; Because process-schema modifies calls the destructive
		    ;; process-node-end.  /\/
		    (s (db-request :GET-NAMED-SCHEMA (cadr task))
		       (deep-copy s))))
	  node-list
	  added-node)
      (when (null schema)
	(dev-debug-msg (format nil "Couldn't get schema for ~S~%" (cadr task))
		       *whoami* :fatal-error)
	(return-from KS-SET-TASK nil))
      ;; Set up an alternative so that if we run out of alternatives, the
      ;; planner gracefully gives up.
      (post-alternatives (make-agenda-rec
			  :body '(:NO-MORE-ALTERNATIVES)
			  :trigger t))
      
      ;; Now process the task schema.
      (dolist (node (schema-nodes schema))
	(if (null (setq added-node
			(db-request :ADD-INITIAL-NODE node)))
	    (progn
	      (dev-debug-msg (format nil "Couldn't add ~S~%" node)
			     *whoami* :fatal-error)
	      (return-from KS-SET-TASK nil)))
	(push (cons (car node) added-node) node-list)
	(if (eql (second node) 'action)
	    (post-expand `(:EXPAND ,added-node ,(node-pattern node)))))
      
      (dolist (link (schema-orderings schema))
	(if (null (db-request :ADD-LINK
			      (list (cdr (assoc (second link) node-list))
				    (third link))
			      (list (cdr (assoc (fourth link) node-list))
				    (fifth link))))
	    (progn
	      (dev-debug-msg (format nil "Couldn't add link ~S~%" link)
			     *whoami* :fatal-error)
	      (return-from KS-SET-TASK nil))))

      (process-schema schema 'NODE t)
      ;; Post an agenda entry to tell the :LEFTOUT channel when the planner
      ;; has finished, so the trigger is :EMPTY.
      (post-agenda '(:PLANNER-FINISHED)
	  :trigger '(:EMPTY)))))

;;;; ---------------------------- Change History ----------------------------
;;;; (Initials) (Date of change)           (Comment)
