;;;; File: oplan.lsp
;;; SCCS Version: %W%
;;; Contains: Initial setup
;;; Author: Jeff Dalton <J.Dalton@ed.ac.uk>
;;; Created: February 1993
;;; Updated: Wed Jul 12 18:16:36 1995 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1993, 1994 AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

;;; Load this file to prepare for compiling or loading O-Plan.
;;; It loads defsystem and, if needed, defpackage, and takes some
;;; other steps to prepare Lisp for what's to come.

;;; Assumes the current directory is "source".

(in-package :user)

(defvar *oplan-version* "2.3")

(push :oplan *features*)

(load "oplan-release-date.lsp")		;defines *oplan-release-date*


;;; AKCL initialization

#+akcl (use-fast-links nil)
; #+akcl (proclaim '(optimize (safety 2)))
#+akcl (proclaim '(optimize (safety 0)))
#+akcl (setq si::*multiply-stacks* 4)

#+akcl (unintern 'lisp)			;/\/ where did LISP come from?

#+akcl (setq *load-verbose* nil)	;/\/ cannot otherwise suppress some
					;    output from loading .o files.


;;; Lucid CL (LCL) initialization

#+:lucid
(progn

  #+lucid
  (proclaim '(optimize (compilation-speed 0) 	;production mode
	               (speed 2)		;tail-merge off
		       (safety 0)))		;no arity or r/w arg checks

  #+lucid-development
  (proclaim '(optimize (compilation-speed 3)
	               (speed 0)
	               (safety 3)))

  (setq lcl:*redefinition-action* :warn) 	; or :query

  (push "lsp" lcl:*load-source-pathname-types*)

  #+solaris
  (setq lcl:*load-binary-pathname-types* (list "s2bin"))

  (setq *load-verbose* nil)

  ;; Avoid conflicts with, e.g., lcl:monitor.
  (setq lcl:*default-make-package-use-list*
	(list (find-package "LISP")))
  (unuse-package :lcl)

  ;; Arrange for some CL additions to work:

  (import '(lcl:defpackage lcl:restart-case lcl:continue
	    lcl:destructuring-bind)
	  :lisp)

  (export '(lcl:defpackage lcl:restart-case lcl:continue
	    lcl:destructuring-bind)
	  :lisp)

); end #+:lucid

;;; Allegro 4.1 initialization

;;; /\/: This does *not* mean that O-Plan works in Allegro CL.
;;; /\/: All it means is that I've started to look at porting it.

#+:allegro
(progn
  #-:allegro-v4.2
  (cerror "Continue anyway." "Expected Allegro version 4.2")
  (push (make-pathname :type "lsp")
	(cddr sys:*load-search-list*))
  (setq excl:*cltl1-in-package-compatibility-p* t)
  (rename-package :ipc :allegro-ipc)
  ;; For some reason, there's a symbol DATE in the USER pkg.
  ;; It conflicts w/ an export of the TF pkg.
  (unintern 'date)
  ;; We'll define SATISFIES and TYPE as functions.
  #+:undef
  (setf (excl:package-definition-lock (find-package :common-lisp))
	nil)
  )


;;; General prep

(unless (fboundp (find-symbol "DEFPACKAGE"))
  (load "support/defpackage"))

(unless (find-package :simple-defsystem)
  (load "support/defsys"))

(shadowing-import '(simple-defsystem:defsystem
                    simple-defsystem:compile-system
                    simple-defsystem:load-system
		    simple-defsystem:clean-system
		    simple-defsystem:print-system-tree))


;;; Package setup

(defpackage "OPLAN"
  (:nicknames "KNOWLEDGE-PLATFORM"
              "KNOWLEDGE-SOURCE")
  (:use "LISP"
        "SIMPLE-DEFSYSTEM"))


;;; System definitions

(load "system-definitions")


;;; KS preloader

(defun preload-knowledge-sources ()
  (load-system 'knowledge-sources :recursive nil)
  (dolist (ks (simple-defsystem:system-files
	       (simple-defsystem:find-system 'knowledge-sources)))
    ;; The names look like "KS-WHATEVER", the keywords like :WHATEVER.
    (let* ((ks-name (simple-defsystem:file-name ks))
	   (keyword (intern (subseq ks-name 3) :keyword)))
      (push keyword oplan::*loaded-ks*)))
  t)

;;; End
