;;;; File: components.lsp
;;; SCCS Version: %W%
;;; Contains: Definition of the component type
;;; Author: Jeff Dalton <J.Dalton@ed.ac.uk>
;;; Created: February 1993
;;; Updated: Fri Oct 21 14:22:51 1994 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1992, 1993, AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

;;; Component defintion

(in-package :components)	;/\/ do we need/want a separate package?

(use-package :pprocess)

(export '(component
	  *component-specials*	;/\/ do we need to export this?
	  new-component
	  component-name
	  component-p
	  component-status
	  component-run-function
	  component-specials
	  component-directory
	  component-debug-level
	  component-init-fn
	  component-interrupt-fn
	  component-single-step-fn))

(export '(*clock*))

(export '(*pprocess*))

(defvar *component-specials*
  '(lisp:*debug-io*
    lisp:*trace-output* 
    lisp:*error-output*
    dev:*whoami*
    dev:*dev-debug-level*
    ctxt:*cucontext*
    util::*pop-gensymtable*
    *clock*
    ))

(defstruct (component (:constructor %make-component)
		      (:print-function print-pprocess)
		      (:include pprocess
			 (specials *component-specials*)))
  (directory (error "No directory for component."))
  (debug-level 1)
  (init-fn nil)
  (interrupt-fn nil)
  (single-step-fn nil))

(defun new-component (name &rest other-initargs)
  (let ((c (apply #'%make-component :name name other-initargs)))
    (register-pprocess c)
    c))

;;; End
