;;;; File: t-frame-tests
;;; SCCS Version: %W%
;;; Contains: Tests of the test framework
;;; Author: Jeff Dalton <J.Dalton@ed.ac.uk>
;;; Created: June 1994
;;; Updated: Sat Jun 11 22:30:15 1994 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1994, AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

(in-package :test-support)

(define-test-module :t-frame-tests)
(in-test-module :t-frame-tests)

(defvar *wrapper-to-test* 'standard-test-wrapper)

(define-test-wrapper :t-frame-tests (test)
  (funcall *wrapper-to-test* test))

;;; These tests should normally be the first ones done.

(define-test-group test-result-tests
  ('a ==> 'a)
  ('a ==> :any)
  ('a)					;same as ==> :any
  ('a ==> 'a :errors 0)
  ('a ==> 'a :errors :any)
  ('a ==> :any :errors :any)
  ('a :errors :any)			;assumes ==> :any

  ;; :True and :false
  ('a ==> :true)
  (10 ==> :true)
  (nil ==> :false))

(define-test-group test-predicate-tests
  ;; The :test predicate is called with the result as its first
  ;; argument and the expected result as its second argument.
  ('a  ==> 'a  :test #'eq)
  ("a" ==> "A" :test #'string-equal)
  ('a  ==> 'symbol :test #'typep))

(define-test-group test-error-tests
  ((error "Test") :errors 1)
  ((error "Test") :errors :some)
  ((error "Test") ==> 'simple-error :test #'typep :errors 1))

(defvar *-a-* nil)

(define-test-group (test-group-with-tests
		      :with ((*-a-* 10) (a 'apple) (b 20)))
  ((boundp '*-a-*)       ==> :true)
  ((symbol-value '*-a-*) ==> 10)
  (*-a-* ==> 10)
  (  a   ==> 'apple)
  (  b   ==> 20))

(define-test-group test-module-tests
  ((exists-test-module :all-tests)     ==> :true)
  ((exists-test-module :t-frame-tests) ==> :true)
  ((test-module-p (exists-test-module :all-tests))     ==> :true)
  ((test-module-p (exists-test-module :t-frame-tests)) ==> :true)
  ((eq (exists-test-module :all-tests)
       (find-test-module :all-tests))
   ==> :true)
  ((eq (exists-test-module :t-frame-tests)
       (find-test-module :t-frame-tests))
   ==> :true)
  ((eq (find-test-module :all-tests)
       (car *all-test-modules*))
   ==> :true)
  ((eq (find-test-module :all-tests)
       (car (last (tm-descendants (find-test-module :all-tests)))))
   ==> :true)
  ((member (find-test-module :t-frame-tests)
	   (tm-descendants (find-test-module :all-tests)))
   ==> :true)
  ((set-eql *all-test-modules*
	    (tm-descendants (find-test-module :all-tests)))
   ==> :true)
  )

(define-test-group test-wrapper-existence-tests
  ((tm-test-wrapper (find-test-module :all-tests))
   ==> 'standard-test-wrapper)
  ((tm-test-wrapper (find-test-module :t-frame-tests))
   ==> 't-frame-tests-test-wrapper))

;;; /\/: Should find a way to test get-test-wrapper too.

(define-test-group (test-wrapper-tests
		    :with ((*wrapper-to-test*
			    #'(lambda (test)
				(let ((*a* 10))
				  (declare (special *a*))
				  (funcall #'standard-test-wrapper test))))))
    ((locally (declare (special *a*))
       *a*)
     ==> 10)
  )

;;; ---------------------------- Change History ----------------------------
;;; (Who)   (When) 		     (What)
;;;
