;;;; File: separate-world.lsp
;;; SCCS Version: %W%
;;; Contains: Code for starting a separate world simulator from the Exec.
;;; Author: Jeff Dalton <J.Dalton@ed.ac.uk>
;;; Created: November 1994
;;; Updated: Thu Jun 22 18:13:14 1995 by Jeff Dalton
;;; Release Version: %Y%
;;; Copyright: (c) 1994, AIAI, University of Edinburgh
;;; This material may be reproduced by or for the U.S. Government pursuant
;;; to the copyright license under the clause at DFARS 252.227-7032
;;; (June 1975) -- Rights in Technical Data and Computer Software (Foreign).

(in-package :world)

(defvar *world-io* nil)

(defun start-separate-world ()
  (setq *world-io* (run-separate-world))
  (new-connector :world-io
    :io-stream *world-io*
    :socket-to-read :worldin
    :socket-to-write :worldout
    :input-guard 'guard-rightin-from-separate-world)
  )

(defun run-separate-world ()
  (let ((shell-command (get-parameter :separate-world)))
    (assert (stringp shell-command))
    (destructuring-bind (cmd &rest args) (break-args shell-command)
      (apply #'ipc-run-agent
	 :world
         (if (find #\/ cmd)
	     cmd
	   (concat-string (get-parameter :oplan-dir) "/bin/" cmd))
	 "-connect"
	 args))))

(defun guard-rightin-from-separate-world (self input)
  ;; Should look like (:WORLD-EVENT (id . args))
  ;; /\/: Another reason to put the simple matcher in the support system.
  (declare (ignore self))
  (if (and (consp input) (keywordp (car input)))
      #+:undef
      (and (consp input)
	   (eq (car input) :world-event)
	   (consp (cadr input))
	   (symbolp (caadr input)))
      (values t input)
    (error "Invalid input from separate world: ~S" input)))

;;; ---------------------------- Change History ----------------------------
;;; (Who)   (When) 		     (What)
;;;
